from IPython.display import IFrame, display, HTML
from ensure import ensure_annotations
import urllib.request
from HTMLrenderer.custom_exception import InvalidURLException
from HTMLrenderer.logger import logger


@ensure_annotations
def is_valid(URL: str) -> bool:
    try:
        response_status = urllib.request.urlopen(URL).getcode()
        print(f"response_status: {response_status} OK")
        assert response_status == 200
        return True
    except Exception:
        return False


@ensure_annotations
def render_site(URL: str = None, width: str = "100%", height: str = "600") -> str:
    """Renders HTML in the jupyter notebook

    Args:
        URL (str): URL of site to render in jupyter notebook. Defaults to None.
        width (str, optional): width of the html page to render_site. Defaults to "100%".
        height (str, optional): height of the html page to render. Defaults to 600.

    Returns:
        None
    """
    try:
        if is_valid(URL):
            response = IFrame(src=URL, width=width, height=height)
            display(response)
            return "success"
        else:
            raise InvalidURLException
    except Exception as e:
        raise e


@ensure_annotations
def get_id_and_start_time(URL: str = None) -> tuple:
    """get youtube video id

    Args:
        URL (str, optional): youtube URL. Defaults to None.

    Returns:
        str: video id
    """

    def _verify_vid_id_len(vid_id, __expected_vid_id_len=11):
        len_of_vid_id = len(vid_id)
        if len_of_vid_id != __expected_vid_id_len:
            raise InvalidURLException(
                f"Invalid video id with length: {len_of_vid_id}, expected {__expected_vid_id_len}"
            )

    logger.info(f"input URL: {URL}")
    split_val = URL.split("=")
    if "watch" in URL:
        if "&t" in URL:
            vid_id, time = split_val[-2][:-2], split_val[-1][:-1]
            _verify_vid_id_len(vid_id)
            logger.info(
                f"vid id: {vid_id}, and starts at: {time}, len of video id: {len(vid_id)}"
            )
            return vid_id, time
        else:
            vid_id, time = split_val[-1], "0"
            _verify_vid_id_len(vid_id)
            logger.info(
                f"vid id: {vid_id}, and starts at: {time}, len of video id: {len(vid_id)}"
            )
            return vid_id, time
    vid_id, time = URL.split("/")[-1], "0"
    _verify_vid_id_len(vid_id)
    logger.info(
        f"vid id: {vid_id}, and starts at: {time}, len of video id: {len(vid_id)}"
    )
    return vid_id, time


@ensure_annotations
def error_playing_video(
    URL: str = None,
    pattern: str = '"playabilityStatus":{"status":"ERROR","reason":"Video unavailable"',
) -> bool:
    request = urllib.request.urlopen(URL)
    return pattern in str(request.read())


@ensure_annotations
def render_YouTube_video(URL: str = None, width: int = 780, height: int = 600):
    """render Youtube videos in notebook

    Args:
        URL (str, optional): Youtube video links. Defaults to None.
        width (int, optional): width of the Youtube video to render. Defaults to 780.
        height (int, optional): height of the Youtube video to render. Defaults to 600.
    Raises:
        e: Exception if youtube link is not valid
    """
    try:
        any_error = error_playing_video(URL)
        if any_error:
            raise InvalidURLException("URL is not accessible")
        if URL is None:
            raise InvalidURLException("URL is None")
        else:
            vid_ID, time = get_id_and_start_time(URL)
            embed_URL = f"https://www.youtube.com/embed/{vid_ID}?start={time}"
            any_error = error_playing_video(embed_URL)
            if not any_error:
                logger.info(f"embed_URL: {embed_URL}")
                iframe = f"""<iframe
                width="{width}" height="{height}"
                src="{embed_URL}"
                title="YouTube video player"
                frameborder="0"
                allow="accelerometer;
                autoplay; clipboard-write;
                encrypted-media; gyroscope;
                picture-in-picture" allowfullscreen>
                </iframe>"""
                display(HTML(iframe))
                return "success"
    except Exception as e:
        raise e



import pytest
from HTMLrenderer import render_YouTube_video
from HTMLrenderer.custom_exception import InvalidURLException


URL_test_success_data = [
    ("https://youtu.be/roO5VGxOw2s", "success"),
    ("https://www.youtube.com/watch?v=roO5VGxOw2s", "success"),
    ("https://www.youtube.com/watch?v=roO5VGxOw2s&t=42s", "success"),
]

URL_test_bad_data = [
    ("https://www.youtube.com/watch?v=roO5VGxOw2sahesbf"),
    ("https://www.youtube.com/watch?v=roO5VGxOw00"),
    ("https://www.youtube.com/watch?v=roO5VGxOw__"),
    ("https://www.youtube.com/watch?v=roO5VGxOwpp"),
    ("https://www.youtube.com/watch?v=roO5VGxOw2s&t"),
    ("https://www.youtube.com/watch?v=roO5VGxOw2s&t==22s"),
    ("https://www.youtube.com/watch?v==roO5VGxOw2s&t=22s"),
]


@pytest.mark.parametrize("URL, response", URL_test_success_data)
def test_render_YT_success(URL, response):
    assert render_YouTube_video(URL) == response


@pytest.mark.parametrize("URL", URL_test_bad_data)
def test_render_YT_failed(URL):
    with pytest.raises(InvalidURLException):
        render_YouTube_video(URL)



import os
import sys
import logging
import inspect

logging_str = "[%(asctime)s: %(levelname)s: %(module)s]: %(message)s"
log_dir = "logs"
log_filepath = os.path.join(log_dir, "running_logs.log")
os.makedirs(log_dir, exist_ok=True)

logging.basicConfig(
    level=logging.INFO,
    format=logging_str,
    handlers=[
        logging.FileHandler(log_filepath),  # , mode="a"),
        logging.StreamHandler(sys.stdout),
    ],
)

s = inspect.stack()
name = os.path.normpath(s[0].filename).split(os.sep)[-2]
logger = logging.getLogger(name)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

__version__ = "0.1.1"

## edit below variables as per your requirements -
REPO_NAME = "HTMLrenderer-c17hawke" # github repo name
AUTHOR_USER_NAME = "c17hawke" # update as per your need
SRC_REPO = "HTMLrenderer" # example sklearn etc
LIST_OF_REQUIREMENTS = [
        'ipython',
        'ipywidgets'
    ] # core dependencies

setuptools.setup(
    name="HTMLrenderer",
    version=__version__,
    author="c17hawke",
    author_email="sunny.c17hawke@gmail.com",
    description="A small package for rendering HTML pages in jupyter notebooks",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}",
    project_urls={
        "Bug Tracker": f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src", exclude="tests"),
    python_requires=">=3.6",
    install_requires=LIST_OF_REQUIREMENTS
)



import pytest
from HTMLrenderer import render_site
from HTMLrenderer.custom_exception import InvalidURLException


URL_test_success_data = [
    ("http://pytorch.org", "success"),
    ("https://pytorch.org", "success"),
]

URL_test_bad_data = [
    ("http://pytorch"),
    ("http//pytorch"),
    ("http:/pytorch"),
    ("http/pytorch"),
    ("http/pytorch"),
    ("pytorch.org"),
]


@pytest.mark.parametrize("URL, response", URL_test_success_data)
def test_render_site_success(URL, response):
    assert render_site(URL) == response


@pytest.mark.parametrize("URL", URL_test_bad_data)
def test_render_site_failed(URL):
    with pytest.raises(InvalidURLException):
        render_site(URL)
